﻿''' <summary>
''' Defines a key-value pair and it's optional comment used in ini file sections for ini files.
''' </summary>
<Serializable()> _
Public Class IniFileKeyValuePair
    Inherits IniKeyValuePairDescription

    ''' <summary>
    ''' Occurs when the key changes.
    ''' </summary>
    Public Event KeyChanged As EventHandler
    ''' <summary>
    ''' Occurs when the value changes.
    ''' </summary>
    Public Event ValueChanged As EventHandler
    ''' <summary>
    ''' Occurs when the comment changes.
    ''' </summary>
    Public Event CommentChanged As EventHandler

    Private ifsIniFileSection As IniFileSection

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class.
    ''' </summary>
    Public Sub New()
        MyBase.New(DirectCast(Nothing, String))
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class with the specified key.
    ''' </summary>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    Public Sub New(ByVal key As String)
        MyBase.New(key)
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class with the specified key and value.
    ''' </summary>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    ''' <param name="value">Specifies the value of the key-value pair.</param>
    Public Sub New(ByVal key As String, ByVal value As String)
        MyBase.New(key, value)
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class with the specified key, value and comment.
    ''' </summary>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    ''' <param name="value">Specifies the value of the key-value pair.</param>
    ''' <param name="comment">Specifies a comment that describes the key-value pair.</param>
    Public Sub New(ByVal key As String, ByVal value As String, ByVal comment As String)
        MyBase.New(key, value, comment)
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class from the specified key-value pair description.
    ''' </summary>
    ''' <param name="description">Specifies the description from which to create the ini file key-value pair.</param>
    Public Sub New(ByVal description As IniKeyValuePairDescription)
        MyBase.New(description.Key, description.Value, description.Comment)
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileSection class and sets its parent to the specified ini file section.
    ''' </summary>
    ''' <param name="section">Specifies the parent section of the key-value pair.</param>
    Public Sub New(ByVal section As IniFileSection)
        MyBase.New(String.Empty)
        ParentSection = section
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileSection class with the specified key and sets its parent to the specified ini file section.
    ''' </summary>
    ''' <param name="section">Specifies the parent section of the key-value pair.</param>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    Public Sub New(ByVal section As IniFileSection, ByVal key As String)
        MyBase.New(key)
        ParentSection = section
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileSection class with the specified key and value and sets its parent to the specified ini file section.
    ''' </summary>
    ''' <param name="section">Specifies the parent section of the key-value pair.</param>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    ''' <param name="value">Specifies the value of the key-value pair.</param>
    Public Sub New(ByVal section As IniFileSection, ByVal key As String, ByVal value As String)
        MyBase.New(key, value)
        ParentSection = section
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileSection class with the specified key, value and comment and sets its parent to the specified ini file section.
    ''' </summary>
    ''' <param name="section">Specifies the parent section of the key-value pair.</param>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    ''' <param name="value">Specifies the value of the key-value pair.</param>
    ''' <param name="comment">Specifies a comment that describes the key-value pair.</param>
    Public Sub New(ByVal section As IniFileSection, ByVal key As String, ByVal value As String, ByVal comment As String)
        MyBase.New(key, value, comment)
        ParentSection = section
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileSection class from the specified description and sets its parent to the specified ini file section.
    ''' </summary>
    ''' <param name="section">Specifies the parent section of the key-value pair.</param>
    ''' <param name="description">Specifies the description from which to create the ini file key-value pair.</param>
    Public Sub New(ByVal section As IniFileSection, ByVal description As IniKeyValuePairDescription)
        MyBase.New(description.Key, description.Value, description.Comment)
        ParentSection = section
    End Sub

    Friend Sub SetParentSection(ByVal value As IniFileSection)
        If ifsIniFileSection IsNot Nothing Then
            Dim ifsv As IniFileSection = Me.ifsIniFileSection
            ifsIniFileSection = value
            ifsv.KeyValues.Remove(Me)
        Else
            ifsIniFileSection = value
        End If
    End Sub

    ''' <summary>
    ''' Gets or sets the section that contains the ini key-value pair.
    ''' </summary>
    Public Property ParentSection As IniFileSection
        Get
            Return ifsIniFileSection
        End Get
        Set(ByVal value As IniFileSection)
            If ifsIniFileSection IsNot Nothing Then
                ifsIniFileSection.KeyValues.Remove(Me)
            End If
            ifsIniFileSection = value
            If Not (value Is Nothing OrElse ifsIniFileSection.KeyValues.Contains(Me)) Then
                ifsIniFileSection.KeyValues.Add(Me)
            End If
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the key-value pair's key.
    ''' </summary>
    ''' <remarks>If the key is empty or null the key-value pair is skiped. Therefore a pure comment or an empty line can be added to the ini file.</remarks>
    Public Overrides Property Key As String
        Get
            Return MyBase.Key
        End Get
        Set(ByVal value As String)
            If ifsIniFileSection IsNot Nothing AndAlso ifsIniFileSection.File IsNot Nothing Then
                If (ifsIniFileSection.File.Flags And IniFileFlags.SupportsKeyValuePairKeyRepetition) <> IniFileFlags.SupportsKeyValuePairKeyRepetition Then
                    If ifsIniFileSection.KeyValues.Contains(value) Then
                        Throw New ArgumentException("The specified key is already present in the ini section. Multiple key-value pairs that have the same key are not allowed by the ini file.")
                    End If
                End If
            End If
            MyBase.Key = value
            OnKeyChanged(New EventArgs())
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the key-value pair's value.
    ''' </summary>
    Public Overrides Property Value As String
        Get
            Return MyBase.Value
        End Get
        Set(ByVal value As String)
            MyBase.Value = value
            OnValueChanged(New EventArgs())
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the comment description of the key-value pair.
    ''' </summary>
    ''' <remarks>Set to null if the comment should not be appended to the key-value pair. If the comment is empty but not null a comment separator is appended.</remarks>
    Public Overrides Property Comment As String
        Get
            Return MyBase.Comment
        End Get
        Set(ByVal value As String)
            MyBase.Comment = value
            OnCommentChanged(New EventArgs())
        End Set
    End Property

    ''' <summary>
    ''' Raises the CommentChanged event.
    ''' </summary>
    ''' <param name="e">An EventArgs that contains the event data.</param>
    Protected Overridable Sub OnCommentChanged(ByVal e As EventArgs)
        RaiseEvent CommentChanged(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the ValueChanged event.
    ''' </summary>
    ''' <param name="e">An EventArgs that contains the event data.</param>
    Protected Overridable Sub OnValueChanged(ByVal e As EventArgs)
        RaiseEvent ValueChanged(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the KeyChanged event.
    ''' </summary>
    ''' <param name="e">An EventArgs that contains the event data..</param>
    Protected Overridable Sub OnKeyChanged(ByVal e As EventArgs)
        RaiseEvent KeyChanged(Me, e)
    End Sub

    ''' <summary>
    ''' Gets a string representation of this key-value pair.
    ''' </summary>
    Public Overrides Function ToString() As String
        If IsEmpty() Then
            Return String.Empty
        ElseIf IsComment() Then
            If ifsIniFileSection Is Nothing Then
                Return ";"c & Comment
            Else
                Return ifsIniFileSection.File.CommentSeparator & Comment
            End If
        ElseIf String.IsNullOrEmpty(Comment) Then
            If ifsIniFileSection Is Nothing Then
                Return String.Concat(Key, "="c, Value)
            Else
                Return String.Concat(Key, ifsIniFileSection.File.KeyValueSeparator, Value)
            End If
        Else
            If ifsIniFileSection Is Nothing Then
                Return String.Concat(Key, "="c, Value, ";", Comment)
            Else
                Return String.Concat(Key, ifsIniFileSection.File.KeyValueSeparator, Value, ifsIniFileSection.File.CommentSeparator, Comment)
            End If
        End If
    End Function

    ''' <summary>
    ''' Clones the key-value pair.
    ''' </summary>
    Public Overrides Function Clone() As Object
        Return New IniFileKeyValuePair(Me)
    End Function
End Class
