﻿Imports Microsoft.VisualBasic

''' <summary>
''' Defines a key-value pair and it's optional comment as used in general ini file sections.
''' </summary>
<Serializable()> _
Public Class IniKeyValuePairDescription
    Implements ICloneable

    Private strKey As String
    Private strValue As String
    Private strComment As String

    ''' <summary>
    ''' Gets or sets the key-value pair's key.
    ''' </summary>
    ''' <remarks>If the key is empty or null the key-value pair is skiped. Therefore a pure comment or an empty line can be added to the ini file.</remarks>
    Public Overridable Property Key As String
        Get
            Return strKey
        End Get
        Set(ByVal value As String)
            If value IsNot Nothing Then
                For Each c As Char In value
                    If c = Chr(10) OrElse c = Chr(13) Then
                        Throw New IniFileSyntaxException("Line feed chracters are not valid.")
                    ElseIf c = ";"c Then
                        Throw New IniFileSyntaxException("Comment characters are not allowed in keys.")
                    End If
                Next
            End If
            strKey = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the key-value pair's value.
    ''' </summary>
    Public Overridable Property Value As String
        Get
            Return strValue
        End Get
        Set(ByVal value As String)
            If value IsNot Nothing Then
                For Each c As Char In value
                    If c = Chr(10) OrElse c = Chr(13) Then
                        Throw New IniFileSyntaxException("Line feed chracters are not valid.")
                    ElseIf c = ";"c Then
                        Throw New IniFileSyntaxException("Comment characters are not allowed in values.")
                    End If
                Next
            End If
            strValue = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the comment description of the key-value pair.
    ''' </summary>
    ''' <remarks>Set to null if the comment should not be appended to the key-value pair. If the comment is empty but not null a comment separator is appended.</remarks>
    Public Overridable Property Comment As String
        Get
            Return strComment
        End Get
        Set(ByVal value As String)
            If value IsNot Nothing Then
                For Each c As Char In value
                    If c = Chr(10) OrElse c = Chr(13) Then
                        Throw New IniFileSyntaxException("Line feed chracters are not valid.")
                    End If
                Next
            End If
            strComment = value
        End Set
    End Property

    ''' <summary>
    ''' Gets whether the current ini key-value pair is rather a comment than a key-value pair.
    ''' </summary>
    Public Overridable ReadOnly Property IsComment As Boolean
        Get
            Return GetIsComment()
        End Get
    End Property

    ''' <summary>
    ''' Gets whether the current ini key-value pair is rather an empty line than a key-value pair or comment.
    ''' </summary>
    Public Overridable ReadOnly Property IsEmpty As Boolean
        Get
            Return GetIsEmpty()
        End Get
    End Property

    ''' <summary>
    ''' Gets whether the current ini key-value pair is rather a comment or an empty line than a key-value pair.
    ''' </summary>
    Public ReadOnly Property IsCommentOrEmpty As Boolean
        Get
            Return GetIsCommentOrEmpty()
        End Get
    End Property

    ''' <summary>
    ''' Returns whether the key-value pair's comment is visible.
    ''' </summary>
    Public ReadOnly Property IsCommentVisible As Boolean
        Get
            Return (strComment IsNot Nothing)
        End Get
    End Property

    ''' <summary>
    ''' Reeturns whether the current key-value pair is a comment or empty.
    ''' </summary>
    Protected Overridable Function GetIsCommentOrEmpty() As Boolean
        Return String.IsNullOrEmpty(Key)
    End Function

    ''' <summary>
    ''' Returns whether a key-value pair with the specified key, value and comment is a comment or empty.
    ''' </summary>
    Protected Overridable Function GetIsCommentOrEmpty(ByVal key As String, ByVal value As String, ByVal comment As String) As Boolean
        Return String.IsNullOrEmpty(key)
    End Function

    ''' <summary>
    ''' Returns whether the current key-value pair is a comment.
    ''' </summary>
    Protected Overridable Function GetIsComment() As Boolean
        Return String.IsNullOrEmpty(Key) AndAlso Not String.IsNullOrEmpty(Comment)
    End Function

    ''' <summary>
    ''' Returns whether a key-value pair with the specified key, value and comment is a comment.
    ''' </summary>
    Protected Overridable Function GetIsComment(ByVal key As String, ByVal value As String, ByVal comment As String) As Boolean
        Return String.IsNullOrEmpty(key) AndAlso Not String.IsNullOrEmpty(comment)
    End Function

    ''' <summary>
    ''' Returns whether the current key-value pair is empty.
    ''' </summary>
    Protected Overridable Function GetIsEmpty() As Boolean
        Return String.IsNullOrEmpty(Key) AndAlso String.IsNullOrEmpty(Comment)
    End Function

    ''' <summary>
    ''' Returns whether a key-value pair with the specified key, value and comment is empty.
    ''' </summary>
    Protected Overridable Function GetIsEmpty(ByVal key As String, ByVal value As String, ByVal comment As String) As Boolean
        Return String.IsNullOrEmpty(key) AndAlso String.IsNullOrEmpty(comment)
    End Function

    ''' <summary>
    ''' Gets a string representation of this key-value pair.
    ''' </summary>
    Public Overrides Function ToString() As String
        If IsEmpty Then
            Return String.Empty
        ElseIf IsComment Then
            Return ";"c & Comment
        ElseIf String.IsNullOrEmpty(Comment) Then
            Return Key & "="c & Value
        Else
            Return Key & "="c & Value & ";"c & Comment
        End If
    End Function

    ''' <summary>
    ''' Converts the key-value pair into a string using the specified key-value separator and the specified comment-separator.
    ''' </summary>
    ''' <param name="keyValueSeparator">Specifies the character used to separate key and value.</param>
    ''' <param name="commentSeparator">Specifies the character used to separate key-value pair and comment.</param>
    Public Overridable Overloads Function ToString(ByVal keyValueSeparator As String, ByVal commentSeparator As Char) As String
        If IsEmpty() Then
            Return String.Empty
        ElseIf IsComment() Then
            Return commentSeparator & Comment
        ElseIf String.IsNullOrEmpty(Comment) Then
            Return Key & keyValueSeparator & Value
        Else
            Return Key & keyValueSeparator & Value & commentSeparator & Comment
        End If
    End Function

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class with the specified key.
    ''' </summary>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    Public Sub New(ByVal key As String)
        Me.Key = key
        Me.Value = Nothing
        Me.Comment = Nothing
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class with the specified key and value.
    ''' </summary>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    ''' <param name="value">Specifies the value of the key-value pair.</param>
    Public Sub New(ByVal key As String, ByVal value As String)
        Me.Key = key
        Me.Value = value
        Me.Comment = Nothing
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class with the specified key, value and comment.
    ''' </summary>
    ''' <param name="key">Specifies the key of the key-value pair.</param>
    ''' <param name="value">Specifies the value of the key-value pair.</param>
    ''' <param name="comment">Specifies a comment that describes the key-value pair.</param>
    Public Sub New(ByVal key As String, ByVal value As String, ByVal comment As String)
        Me.Key = key
        Me.Value = value
        Me.Comment = comment
    End Sub

    ''' <summary>
    ''' Initializes a new instance of IniLib.IniFileKeyValuePair class from the specified key-value pair description.
    ''' </summary>
    ''' <param name="description">Specifies the description from which to create the ini file key-value pair.</param>
    Public Sub New(ByVal description As IniKeyValuePairDescription)
        Me.Key = description.Key
        Me.Value = description.Value
        Me.Comment = description.Comment
    End Sub

    ''' <summary>
    ''' Loads a key-value pair from the specified character enumeration.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    Public Overridable Sub LoadFromChars(ByVal chars As IEnumerable(Of Char))
        Dim buff As New System.Text.StringBuilder(256)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Throw New IniFileSyntaxException("Comments are not valid in keys.")
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        Key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        Value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        enr.Dispose()
        Comment = buff.ToString()
    End Sub

    ''' <summary>
    ''' Loads a key-value pair from the specified character enumeration into the current instance specifying the default length of the string buffer used.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    ''' <param name="stringBufferDefaultLength">Specifies the default length of the string buffer that is used to buffer key, value and comment.</param>
    Public Overridable Sub LoadFromChars(ByVal chars As IEnumerable(Of Char), ByVal stringBufferDefaultLength As Integer)
        Dim buff As New System.Text.StringBuilder(stringBufferDefaultLength)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Throw New IniFileSyntaxException("Comments are not valid in keys.")
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        Key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        Value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        enr.Dispose()
        Comment = buff.ToString()
    End Sub

    ''' <summary>
    ''' Loads a key-value pair from the specified character enumeration into a new instance of IniKeyValuePairDescription class.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    Public Shared Function FromChars(ByVal chars As IEnumerable(Of Char)) As IniKeyValuePairDescription
        Dim buff As New System.Text.StringBuilder(256)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim key As String
        Dim value As String
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Throw New IniFileSyntaxException("Comments are not valid in keys.")
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        enr.Dispose()
        Return New IniKeyValuePairDescription(key, value, If(commentIsNull, Nothing, buff.ToString()))
    End Function

    ''' <summary>
    ''' Loads a key-value pair from the specified character enumeration into a new instance of IniKeyValuePairDescription class specifying the default length of the string buffer used.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    ''' <param name="stringBufferDefaultLength">Specifies the default length of the string buffer that is used to buffer key, value and comment.</param>
    Public Shared Function FromChars(ByVal chars As IEnumerable(Of Char), ByVal stringBufferDefaultLength As Integer) As IniKeyValuePairDescription
        Dim buff As New System.Text.StringBuilder(stringBufferDefaultLength)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim key As String
        Dim value As String
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Throw New IniFileSyntaxException("Comments are not valid in keys.")
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Throw New IniFileSyntaxException("Line feed characters are not valid in strings.")
            End If
        End While
        enr.Dispose()
        Return New IniKeyValuePairDescription(key, value, If(commentIsNull, Nothing, buff.ToString()))
    End Function

    ''' <summary>
    ''' Tries to parse the specified characters and returns false whenever a syntax error occures.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    ''' <param name="result">If the method returns true this parameter is the target of the parsing process. Otherwise the parameter does not change.</param>
    Public Shared Function TryFromChars(ByVal chars As IEnumerable(Of Char), ByRef result As IniKeyValuePairDescription) As Boolean
        Dim buff As New System.Text.StringBuilder(256)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim key As String
        Dim value As String
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Return False
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Return False
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Return False
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Return False
            End If
        End While
        enr.Dispose()
        result = New IniKeyValuePairDescription(key, value, If(commentIsNull, Nothing, buff.ToString()))
        Return True
    End Function

    ''' <summary>
    ''' Tries to parse the specified characters and returns false whenever a syntax error occures.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    ''' <param name="stringBufferDefaultLength">Specifies the default length of the string buffer that is used to buffer key, value and comment.</param>
    ''' <param name="result">If the method returns true this parameter is the target of the parsing process. Otherwise the parameter does not change.</param>
    Public Shared Function TryFromChars(ByVal chars As IEnumerable(Of Char), ByVal stringBufferDefaultLength As Integer, ByRef result As IniKeyValuePairDescription) As Boolean
        Dim buff As New System.Text.StringBuilder(stringBufferDefaultLength)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim key As String
        Dim value As String
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Return False
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Return False
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Return False
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Return False
            End If
        End While
        enr.Dispose()
        result = New IniKeyValuePairDescription(key, value, If(commentIsNull, Nothing, buff.ToString()))
        Return True
    End Function

    ''' <summary>
    ''' Tries to parse the specified characters and returns false whenever a syntax error occures.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    Public Function TryFromChars(ByVal chars As IEnumerable(Of Char)) As Boolean
        Dim buff As New System.Text.StringBuilder(256)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim key As String
        Dim value As String
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Return False
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Return False
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Return False
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Return False
            End If
        End While
        enr.Dispose()
        Me.Key = key
        Me.Value = value
        Me.Comment = If(commentIsNull, Nothing, buff.ToString())
        Return True
    End Function

    ''' <summary>
    ''' Tries to parse the specified characters and returns false whenever a syntax error occures.
    ''' </summary>
    ''' <param name="chars">Specifies the characters containing the key-value pair.</param>
    ''' <param name="stringBufferDefaultLength">Specifies the default length of the string buffer that is used to buffer key, value and comment.</param>
    Public Function TryFromChars(ByVal chars As IEnumerable(Of Char), ByVal stringBufferDefaultLength As Integer) As Boolean
        Dim buff As New System.Text.StringBuilder(stringBufferDefaultLength)
        Dim enr As IEnumerator(Of Char) = chars.GetEnumerator()
        Dim key As String
        Dim value As String
        Dim commentIsNull As Boolean = True
        Dim numberWhitespaces As Integer
        'Skip whitespaces
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = "="c Then
                Exit While
            ElseIf enr.Current = ";"c Then
                Return False
            ElseIf enr.Current <> Chr(13) AndAlso enr.Current <> Chr(10) Then
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            Else
                Return False
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        key = buff.ToString()
        buff.Remove(0, buff.Length)
        numberWhitespaces = 0
        While enr.MoveNext() AndAlso (enr.Current = " "c OrElse enr.Current = Chr(9))
        End While
        While enr.MoveNext()
            If enr.Current = ";"c Then
                commentIsNull = False
                Exit While
            ElseIf enr.Current = Chr(10) OrElse enr.Current = Chr(13) Then
                Return False
            Else
                If enr.Current = " "c OrElse enr.Current = Chr(9) Then
                    numberWhitespaces += 1
                Else
                    numberWhitespaces = 0
                End If
                buff.Append(enr.Current)
            End If
        End While
        buff.Remove(buff.Length - numberWhitespaces, numberWhitespaces)
        value = buff.ToString()
        buff.Remove(0, buff.Length)
        While enr.MoveNext()
            If enr.Current <> Chr(10) AndAlso enr.Current <> Chr(13) Then
                buff.Append(enr.Current())
            Else
                Return False
            End If
        End While
        enr.Dispose()
        Me.Key = key
        Me.Value = value
        Me.Comment = If(commentIsNull, Nothing, buff.ToString())
        Return True
    End Function

    ''' <summary>
    ''' Clones the key-value pair.
    ''' </summary>
    Public Overridable Function Clone() As Object Implements System.ICloneable.Clone
        If Me.GetType() Is GetType(IniKeyValuePairDescription) Then
            Return New IniKeyValuePairDescription(Me)
        Else
            Throw New NotSupportedException("Cloning method not implemented for the subclass of this ini key-value pair description.")
        End If
    End Function
End Class
